@@ -1,41 +1,43 @@
-import os
 import logging
+import os
 from logging.handlers import RotatingFileHandler
-from loguru import logger as loguru_logger
+
 from converge import settings
+from loguru import logger as loguru_logger
 
 
-def build_api_logger():
+def build_api_logger(rotate=False):
     """
     Builds multiprocess-safe (hence loguru over stdlib logger) API logger
     """
-    level = settings.API_LOGGER.LEVEL
     handler = settings.API_LOGGER.FILEPATH
     if handler:  # Else log to sys.stderr by default
-        rotation = settings.API_LOGGER.ROTATION
-        retention = settings.API_LOGGER.RETENTION
         loguru_logger.add(
             handler,
-            retention=retention,
-            rotation=rotation,
+            retention=settings.API_LOGGER.RETENTION if rotate else None,
+            rotation=settings.API_LOGGER.ROTATION if rotate else None,
             format="{time:YYYY-MM-DD HH:mm:ss} | {message}",
             enqueue=True,
-            level=level,
+            level=settings.API_LOGGER.LEVEL,
         )
     return loguru_logger
 
 
-def build_app_logger(name="app", logfile="app.log", debug=True):
+def build_app_logger(name="app", logfile="app.log", debug=True, rotate=False):
     """
     General purpose application logger. Useful mainly for debugging
     """
     level = logging.DEBUG if settings.DEBUG else logging.INFO
-    logdir = "logs"  # TODO: move this to settings
+    logdir = getattr(getattr(settings, "APP_LOGGER", None), "LOGDIR", "logs")
     if not os.path.exists(logdir):
         os.mkdir(logdir)
     logpath = os.path.join(logdir, logfile)
     maxBytes = 1024 * 1024 * 10
-    handler = RotatingFileHandler(logpath, maxBytes=maxBytes, backupCount=100)
+
+    if rotate:
+        handler = RotatingFileHandler(logpath, maxBytes=maxBytes, backupCount=100)
+    else:
+        handler = logging.FileHandler(logpath)
     handler.setLevel(level)
     formatter = logging.Formatter("[%(levelname)s] %(asctime)s: %(message)s")
     handler.setFormatter(formatter)
@@ -45,5 +47,10 @@ def build_app_logger(name="app", logfile="app.log", debug=True):
     return logger
 
 
-api_logger = build_api_logger() if settings.API_LOGGER.ENABLED else None
 app_logger = build_app_logger()
+app_logger.info(f"Running app in '{settings.APP_MODE}' mode")
+
+api_logger = None
+if settings.API_LOGGER.ENABLED:
+    api_logger = build_api_logger()
+    api_logger.info(f"Running api in '{settings.APP_MODE}' mode")
