@@ -1,3 +1,5 @@
+# type: ignore
+
 from dataclasses import asdict, dataclass
 
 import hug
@@ -5,14 +7,15 @@ from converge import settings
 from falcon import HTTPForbidden, HTTPNotFound, HTTPUnauthorized
 from hug.decorators import wraps
 
-from apphelpers.db.peewee import dbtransaction
-from apphelpers.errors import BaseError, InvalidSessionError
+from apphelpers.db import dbtransaction
+from apphelpers.errors.hug import BaseError, InvalidSessionError
 from apphelpers.loggers import api_logger
+from apphelpers.rest import endpoint as ep
+from apphelpers.rest.common import notify_honeybadger
 from apphelpers.sessions import SessionDBHandler
 
 if settings.get("HONEYBADGER_API_KEY"):
     from honeybadger import Honeybadger
-    from honeybadger.utils import filter_dict
 
 
 def phony(f):
@@ -20,7 +23,7 @@ def phony(f):
 
 
 def raise_not_found_on_none(f):
-    if getattr(f, "not_found_on_none", None) == True:
+    if getattr(f, "not_found_on_none", None) is True:
 
         @wraps(f)
         def wrapper(*ar, **kw):
@@ -42,24 +45,25 @@ def honeybadger_wrapper(hb):
     def wrapper(f):
         @wraps(f)
         def f_wrapped(*args, **kw):
+            err_to_report = None
             try:
-                ret = f(*args, **kw)
+                return f(*args, **kw)
             except BaseError as e:
-                # Don't report BaseError
+                if e.report:
+                    err_to_report = e
                 raise e
             except Exception as e:
-                try:
-                    hb.notify(
-                        e,
-                        context={
-                            "func": f.__name__,
-                            "args": args,
-                            "kwargs": filter_dict(kw, settings.HB_PARAM_FILTERS),
-                        },
+                err_to_report = e
+                raise e
+            finally:
+                if err_to_report:
+                    notify_honeybadger(
+                        honeybadger=hb,
+                        error=err_to_report,
+                        func=f,
+                        args=args,
+                        kwargs=kw,
                     )
-                finally:
-                    raise e
-            return ret
 
         return f_wrapped
 
@@ -81,6 +85,21 @@ def user_email(default=None, request=None, **kwargs):
     return request.context["user"].email
 
 
+@hug.directive()
+def user_groups(default=None, request=None, **kwargs):
+    return request.context["user"].groups or tuple()
+
+
+@hug.directive()
+def user_site_groups(default=None, request=None, **kwargs):
+    return request.context["user"].site_groups or {}
+
+
+@hug.directive()
+def user_site_ctx(default=None, request=None, **kwargs):
+    return request.context["user"].site_ctx
+
+
 @hug.directive()
 def domain(default=None, request=None, **kwargs):
     return request.headers["HOST"]
@@ -91,6 +110,11 @@ def user_mobile(default=None, request=None, **kwargs):
     return request.context["user"].mobile
 
 
+@hug.directive()
+def user_agent(default=None, request=None, **kwargs):
+    return request.headers.get("USER-AGENT", "")
+
+
 @dataclass
 class User:
     sid: str = None
@@ -100,6 +124,7 @@ class User:
     email: str = None
     mobile: str = None
     site_groups: dict = None
+    site_ctx: None = None
 
     def to_dict(self):
         return asdict(self)
@@ -110,21 +135,38 @@ class User:
 
 def setup_strict_context_setter(sessions):
     def set_context(token):
-
-        uid, groups, name, email, mobile, site_groups = None, [], "", None, None, {}
+        uid, groups, name, email, mobile, site_groups, site_ctx = (
+            None,
+            [],
+            "",
+            None,
+            None,
+            {},
+            None,
+        )
 
         if token:
             try:
                 session = sessions.get(
-                    token, ["uid", "name", "groups", "email", "mobile", "site_groups"]
+                    token,
+                    [
+                        "uid",
+                        "name",
+                        "groups",
+                        "email",
+                        "mobile",
+                        "site_groups",
+                        "site_ctx",
+                    ],
                 )
-                uid, name, groups, email, mobile, site_groups = (
+                uid, name, groups, email, mobile, site_groups, site_ctx = (
                     session["uid"],
                     session["name"],
                     session["groups"],
                     session["email"],
                     session["mobile"],
                     session["site_groups"],
+                    session["site_ctx"],
                 )
             except InvalidSessionError:
                 raise HTTPUnauthorized("Invalid or expired session")
@@ -137,6 +179,7 @@ def setup_strict_context_setter(sessions):
             email=email,
             mobile=mobile,
             site_groups=site_groups,
+            site_ctx=site_ctx,
         )
 
     return set_context
@@ -148,20 +191,38 @@ def setup_context_setter(sessions):
         Only sets context based on session.
         Does not raise any error
         """
-        uid, groups, name, email, mobile, site_groups = None, [], "", None, None, {}
+        uid, groups, name, email, mobile, site_groups, site_ctx = (
+            None,
+            [],
+            "",
+            None,
+            None,
+            {},
+            None,
+        )
         token = request.get_header("Authorization")
         if token:
             try:
                 session = sessions.get(
-                    token, ["uid", "name", "groups", "email", "mobile", "site_groups"]
+                    token,
+                    [
+                        "uid",
+                        "name",
+                        "groups",
+                        "email",
+                        "mobile",
+                        "site_groups",
+                        "site_ctx",
+                    ],
                 )
-                uid, name, groups, email, mobile, site_groups = (
+                uid, name, groups, email, mobile, site_groups, site_ctx = (
                     session["uid"],
                     session["name"],
                     session["groups"],
                     session["email"],
                     session["mobile"],
                     session["site_groups"],
+                    session["site_ctx"],
                 )
                 if api_logger:
                     api_logger.info("{} | {} | {}", uid, request.method, request.url)
@@ -177,6 +238,7 @@ def setup_context_setter(sessions):
             email=email,
             mobile=mobile,
             site_groups=site_groups,
+            site_ctx=site_ctx,
         )
 
     return set_context
@@ -229,15 +291,21 @@ class APIFactory:
             This is the authentication + authorization part
             """
             login_required = getattr(f, "login_required", None)
-            groups_required = getattr(f, "groups_required", None)
+            any_group_required = getattr(f, "any_group_required", None)
+            all_groups_required = getattr(f, "all_groups_required", None)
             groups_forbidden = getattr(f, "groups_forbidden", None)
             authorizer = getattr(f, "authorizer", None)
 
-            if login_required or groups_required or groups_forbidden or authorizer:
+            if (
+                login_required
+                or any_group_required
+                or all_groups_required
+                or groups_forbidden
+                or authorizer
+            ):
 
                 @wraps(f)
                 def wrapper(request, *args, **kw):
-
                     user = request.context["user"]
 
                     # this is authentication part
@@ -247,7 +315,12 @@ class APIFactory:
                     # this is authorization part
                     groups = set(user.groups)
 
-                    if groups_required and not groups.intersection(groups_required):
+                    if any_group_required and groups.isdisjoint(any_group_required):
+                        raise HTTPForbidden("Unauthorized access")
+
+                    if all_groups_required and not groups.issuperset(
+                        all_groups_required
+                    ):
                         raise HTTPForbidden("Unauthorized access")
 
                     if groups_forbidden and groups.intersection(groups_forbidden):
@@ -268,28 +341,51 @@ class APIFactory:
             This is the authentication + authorization part
             """
             login_required = getattr(f, "login_required", None)
-            groups_required = getattr(f, "groups_required", None)
+            any_group_required = getattr(f, "any_group_required", None)
+            all_groups_required = getattr(f, "all_groups_required", None)
             groups_forbidden = getattr(f, "groups_forbidden", None)
             authorizer = getattr(f, "authorizer", None)
 
-            if login_required or groups_required or groups_forbidden or authorizer:
+            if (
+                login_required
+                or any_group_required
+                or all_groups_required
+                or groups_forbidden
+                or authorizer
+            ):
 
                 @wraps(f)
                 def wrapper(request, *args, **kw):
-
                     user = request.context["user"]
+                    site_id = (
+                        int(kw[self.site_identifier])
+                        if self.site_identifier in kw
+                        else None
+                    )
 
                     # this is authentication part
                     if not user.id:
                         raise HTTPUnauthorized("Invalid or expired session")
 
+                    # bound site authorization
+                    if (
+                        user.site_ctx
+                        and site_id != user.site_ctx
+                        and getattr(f, "ignore_site_ctx", False) is False
+                    ):
+                        raise HTTPUnauthorized("Invalid or expired session")
+
                     # this is authorization part
                     groups = set(user.groups)
-                    if self.site_identifier in kw:
-                        site_id = int(kw[self.site_identifier])
+                    if site_id:
                         groups = groups.union(user.site_groups.get(site_id, []))
 
-                    if groups_required and not groups.intersection(groups_required):
+                    if any_group_required and groups.isdisjoint(any_group_required):
+                        raise HTTPForbidden("Unauthorized access")
+
+                    if all_groups_required and not groups.issuperset(
+                        all_groups_required
+                    ):
                         raise HTTPForbidden("Unauthorized access")
 
                     if groups_forbidden and groups.intersection(groups_forbidden):
@@ -315,11 +411,15 @@ class APIFactory:
 
     def build(self, method, method_args, method_kw, f):
         print(
-            f"{method_args[0]} [{method.__name__.upper()}] => {f.__module__}:{f.__name__}"
+            f"{method_args[0]}",
+            f"[{method.__name__.upper()}] => {f.__module__}:{f.__name__}",
+        )
+        db_tr_wrapper = (
+            phony if getattr(f, "skip_dbtransaction", False) else self.db_tr_wrapper
         )
         m = method(*method_args, **method_kw)
         f = self.access_wrapper(
-            self.honeybadger_wrapper(self.db_tr_wrapper(raise_not_found_on_none(f)))
+            self.honeybadger_wrapper(db_tr_wrapper(raise_not_found_on_none(f)))
         )
         # NOTE: ^ wrapper ordering is important. access_wrapper needs request which
         # others don't. If access_wrapper comes late in the order it won't be passed
@@ -394,3 +494,8 @@ class APIFactory:
             self.patch(resource_url)(update_resource)
         if delete_resource:
             self.delete(resource_url)(delete_resource)
+
+
+@ep.login_required
+def whoami(user: hug.directives.user):
+    return user.to_dict()
