@@ -1,10 +1,11 @@
 import json
+
 import jwt
 import requests
 
 try:
     from converge import settings
-except:
+except ImportError:
     import settings
 
 
