@@ -1,9 +1,10 @@
 import os
+
 from requests_oauthlib import OAuth2Session
 
 try:
     from converge import settings
-except:
+except ImportError:
     import settings
 
 os.environ["OAUTHLIB_INSECURE_TRANSPORT"] = "True"
