@@ -1,6 +1,7 @@
 import hug
-import settings
 
+import settings
+from apphelpers.rest import endpoint as ep
 from apphelpers.rest.hug import APIFactory
 
 
@@ -8,13 +9,11 @@ def echo(word):
     return {"word": word}
 
 
+@ep.login_required
 def protected_echo(word):
     return {"word": word}
 
 
-protected_echo.login_required = True
-
-
 def setup_routes(factory):
     factory.get("/echo/{word}")(echo)
     factory.post("/echo")(echo)
