@@ -1,9 +1,15 @@
-from apphelpers.db.peewee import create_pgdb_pool, create_base_model, dbtransaction
+import pytest
 from peewee import TextField
 
 import settings
+from apphelpers.db.peewee import create_base_model, create_pgdb_pool, dbtransaction
 
-db = create_pgdb_pool(database=settings.DB_NAME, user=settings.DB_USER)
+db = create_pgdb_pool(
+    host=settings.DB_HOST,
+    database=settings.DB_NAME,
+    user=settings.DB_USER,
+    password=settings.DB_PASS,
+)
 BaseModel = create_base_model(db)
 dbtransaction = dbtransaction(db)
 
@@ -18,7 +24,7 @@ def _add_book(name):
 
 def _add_book_loser(name):
     _add_book(name)
-    loser  # will raise
+    loser  # will raise  # noqa: F821
 
 
 def setup_module():
@@ -30,6 +36,8 @@ def teardown_module():
 
 
 def test_add_with_tr():
+    Book.delete().execute()
+
     add_book = dbtransaction(_add_book)
     name = "The Pillars of the Earth"
     add_book(name)
@@ -38,10 +46,8 @@ def test_add_with_tr():
 
     add_book_loser = dbtransaction(_add_book_loser)
     name = "The Cathedral and the Bazaar"
-    try:
+    with pytest.raises(NameError):
         add_book_loser(name)
-    except NameError:
-        pass
     names = [b.name for b in Book.select()]
     assert name not in names
 
